import Taro, { Component } from '@tarojs/taro';
import { ScrollView, View } from '@tarojs/components';
import { TaroCanvasDrawer } from '@components/taro-plugin-canvas';
import {} from './util';
import './style.scss';

export default class Poster extends Component {
  state = {
    images: [],
    current: undefined,
  }

  onScroll = (e) => {
    console.log('---onScroll-->', e)
  }
  
  onPreview = (e) => {
    e.stopPropagation()
    e.preventDefault()
    Taro.previewImage({
      current: this.state.current,
      urls: this.state.images
    })
  }

  onCancel = () => {
    if (typeof this.props.onCancel === 'function') {
      this.props.onCancel()
    }
  }

  saveToAlbum = () => {

  }

  onCreateSuccess = (inx) => (result) => {
    if (inx + 1 === this.props.tpls.length) {
      Taro.hideLoading();
    }

    const { tempFilePath, errMsg } = result;
    if (errMsg === 'canvasToTempFilePath:ok') {
      this.setState({
        shareImage: tempFilePath,
        canvasStatus: false,
        config: null,
        visible: true
      });
    } else {
      console.log(errMsg);
    }

  }

  onCreateFail = (inx) => (err) => {
    if (inx + 1 === this.props.tpls.length) {
      Taro.hideLoading();
    }

    console.err(err);
  }

  render() {
    const {
      images,
      configs
    } = this.state

    return (
      <View className="poster">
        <View className="con">
          <View className="con-head">
            <Text>保存后分享图片</Text>
            <Text className="iconfont icon-buoumaotubiao20" onClick={this.onCancel}></Text>
          </View>
          <View className="con-body">
            {
              images.map((img, inx) => {
                <ScrollView scrollX scrollWithAnimation onScroll={this.onScroll} style="width: 100%" key={`img-${inx}`}>
                  <Image className="result-img" mode="aspectFit" lazy-load src={img} onClick={this.onPreview}></Image>
                </ScrollView>
              })
            }
          </View>
          <Button className="save-btn" onClick={this.saveToAlbum}>保存海报</Button>
        </View>
        {
          configs && configs.length &&
          (
            configs.map((conf, inx) => {
              <TaroCanvasDrawer
                config={conf}
                onCreateSuccess={this.onCreateSuccess(inx)}
                onCreateFail={this.onCreateFail(inx)}
              />
            })
          )
        }
      </View>
    );
  }
}
