import Taro, { Component } from '@tarojs/taro';
import './index.scss'
import Notice from '@components/Notice'
import BackHomeBtn from '@components/BackHomeBtn'
import Poster from './poster'
import dayjs from 'dayjs'
import WxParse from '@components/wxParse/wxParse'
import getUserPhone from '@utils/getUserPhone'
import ready from '@utils/ready'
import { getDownloadURL } from '@utils/tools'
import {
  addActivityShareNum,
  signupActivity,
  favorActivity,
  cancelFavorActivity,
  queryActivityDetail
} from '@services/activity'
import {
  getMiniQrcode,
  savePoint,
  updatePoint
} from '@services/common'
import { connect } from '@tarojs/redux'

@connect(state => state.user)
export default class Detail extends Component {
  config = {
    navigationBarTitleText: '拼团详情'
  }

  state = {
    id: null,
    loaded: false,
    detail: {},
    recordId: null,
    isSaved: false,
    isSign: false,
    posterShow: false,
    posterData: {},
    posterVisible: false,
    selector: ['1', '2', '3'],
    selectorChecked: '1',
    canChoose: 'none',
    inputName: ''
  }

  componentWillMount() {
    ready.queue(() => {
      const router = Taro.getStorageSync('router')
      const id = this.$router.params.id || router.query.id || '2fe920719b8b0db50011441462e647ad'
      this.setState({ id }, () => {
        this.loadDetail()
      })
    })
  }
  componentWillUnmount() {
    const { recordId } = this.state
    recordId && updatePoint(recordId)
  }

  loadDetail() {
    const { id } = this.state
    Taro.showLoading()
    queryActivityDetail(id).then(res => {
      Taro.hideLoading()
      const { isSaved, isSign } = res
      this.setState({
        detail: res,
        isSaved,
        isSign,
        loaded: true
      }, () => {
        const { detail } = this.state
        if (detail.posters[0].posterImg) {
          this.setState({
            posterShow: true
          })
        }
      })
      WxParse.wxParse('article', 'html', res.desc, this.$scope, 0)

      savePoint({
        event: 'detail',
        eventType: 'activity',
        propertyName: '活动详情',
        buildingId: res.buildingId,
        targetId: res.dynamicId,
        data: '{}'
      }).then(res1 => {
        this.setState({
          recordId: res1.recordId
        })
        console.log('活动详情')
      })
    })
  }
  onShareAppMessage = () => {
    const { detail: { shareContents, title, dynamicId, url } } = this.state
    const { userInfo: { person: { personId } } } = this.props

    addActivityShareNum(dynamicId)

    savePoint({
      event: 'share',
      eventType: 'activity',
      propertyName: '活动详情分享',
      data: '{}'
    }).then(res => {
      console.log('活动详情分享')
    })
    return {
      title: shareContents[0].shareContentTitle,
      path: `/pages/activity/detail/index?id=${dynamicId}&from=dynamic_share&recommender=${personId}`,//分享地址
      imageUrl: shareContents[0].shareContentImg
    }
  }

  handleSignup() {
    const { detail: { buildingId, dynamicId }, isSign } = this.state
    const { userInfo: { person: { phone, name, nickname } } } = this.props

    if (isSign) {
      Taro.showToast({
        icon: 'none',
        title: '你已报名成功'
      })
      return
    }

    this.setState({
      canChoose: 'block'
    })
  }

  handleFavor() {
    const { detail: { dynamicId }, isSaved } = this.state
    if (isSaved) {
      cancelFavorActivity(dynamicId).then(res => {
        Taro.showToast({
          title: '已取消收藏'
        })
        this.setState({
          isSaved: false
        })
      })
    } else {
      favorActivity(dynamicId).then(res => {
        Taro.showToast({
          title: '收藏成功'
        })
        this.setState({
          isSaved: true
        })
      })
    }
    savePoint({
      event: 'save',
      eventType: 'activity',
      propertyName: '活动详情收藏',
      data: '{}'
    }).then(res => {
      console.log('活动详情收藏')
    })
  }

  handleGetPhone(e) {
    getUserPhone(e, (phoneNumber) => {
      if (phoneNumber) {
        this.handleSignup()
      } else {
        Taro.showToast({
          title: '报名失败',
          icon: 'none'
        })
      }
    })
  }
  getPosterData = () => {
    return new Promise(resolve => {
      const { posterData } = this.state
      if (posterData.qrcode) {
        resolve(posterData)
        return
      }

      const { userInfo: { person } } = this.props
      const { avatarurl, nickname, personId } = person
      const { detail: { dynamicId, posters, createDate } } = this.state
      const payload = {
        "scene": `id=${dynamicId}&from=dynamic_share&recommender=${personId}`,
        "page": 'pages/activity/detail/index',
      }
      debugger
      const _avatarurl = getDownloadURL(avatarurl, 'avatar');
      getMiniQrcode(payload).then(qrcode => {
        let data = {
          qrcode,//小程序二维码
          nickname,//访问者名字
          avatarurl: _avatarurl,//访问者头像
          poster: posters[0].posterImg,
          title: posters[0].posterTitle,//海报标题
          createDate: createDate,//时间
        }
        resolve(data)
      })
    })
  }
  // 开始生成海报
  togglePosterStatus = (flag) => {
    if (flag) {
      this.getPosterData().then(posterData => {
        this.setState({
          posterVisible: !!flag,
          posterData
        })
      })
      // App.zhuge.track('生成活动详情海报')
    } else {
      this.setState({
        posterVisible: !!flag
      })
    }
    savePoint({
      event: 'poster',
      eventType: 'activity',
      propertyName: '生成活动详情海报',
      data: '{}'
    }).then(res => {
      console.log('生成活动详情海报')
    })
  }

  handleMoreClick() {
    // App.zhuge.track('查看置业顾问列表')
    console.log()
    Taro.navigateTo({
      url: `/pages/card/list/index?buildingId=${this.$router.params.buildingId}`
    })
  }

  onChange = e => {
    this.setState({
      selectorChecked: this.state.selector[e.detail.value]
    })
  }

  onInputText = e => {
    this.setState({
      inputName: e.detail.value
    })
  }

  comfire = e => {

    const { detail: { buildingId, dynamicId }, isSign, selectorChecked, inputName } = this.state
    const { userInfo: { person: { phone, name, nickname, tel } } } = this.props

    if (inputName == '') {
      Taro.showToast({
        title: '请输入姓名',
        icon: 'none'
      })
      return
    }

    const payload = {
      buildingId,
      dynamicId,
      name: inputName,
      phone: phone ? phone : tel,
      attendNum: selectorChecked
    }

    signupActivity(payload).then(res => {
      Taro.showToast({
        title: '报名成功'
      })
      this.setState({
        isSign: true,
        canChoose: 'none'
      })
    })

    savePoint({
      event: 'enlist',
      eventType: 'activity',
      propertyName: '活动详情报名',
      data: '{}'
    }).then(res => {
      console.log('活动详情报名 ')
    })
  }

  hideModal() {
    this.setState({
      canChoose: 'none'
    })
  }

  render() {
    const { detail, loaded, isSaved, isSign, posterVisible, posterData, posterShow } = this.state
    const { userInfo: { person: { phone } } } = this.props
    return (
      <Block>
        {/* 生成海报 */}
        {posterVisible && (<Poster data={posterData} toggle={this.togglePosterStatus}></Poster>)}

        {
          loaded && (
            <View>
              <Notice></Notice>
              <View className="detail-banner">
                <Image mode="widthFix" src={detail.imgUrl} className="detail-banner__img"></Image>
                <View className="rest-time">
                  <Text className="row-label">活动剩余时间：</Text>
                  <Text className="row-content">{dayjs(detail.enlistEnd).format('YYYY-MM-DD hh:mm:ss')}</Text>
                </View>
              </View>
              <ScrollView
                scrollY
                className="detail-wrap">
                <View className="detail">
                  <View class="detail-title">{detail.title}</View>
                  <View className="detail-info">
                    <View style="margin-right:30px">所需<Text style="color:#FF245A">300</Text>积分</View>
                    <Image style="width:20px;height:20px;margin-right:2px" src={require('@assets/mine/regiment.png')} ></Image>
                    <Text >5人团</Text>
                  </View>

                  <Image src={require('@assets/shop/banner1.jpg')} className="my__img"></Image>

                  <View className='tools-main'>
                    <View className='tools-item'>
                      <Image src={require('@assets/shop/banner1.jpg')} className='tools-item-img'></Image>
                      <Text className='tools-item-txt'>等待拼团</Text>
                    </View>
                    <View className='tools-item'>
                      <Image src={require('@assets/shop/banner1.jpg')} className='tools-item-img'></Image>
                      <Text className='tools-item-txt'>等待拼团</Text>
                    </View>
                    <View className='tools-item'>
                      <Image src={require('@assets/shop/banner1.jpg')} className='tools-item-img'></Image>
                      <Text className='tools-item-txt'>等待拼团</Text>
                    </View>
                    <View className='tools-item'>
                      <Image src={require('@assets/shop/banner1.jpg')} className='tools-item-img'></Image>
                      <Text className='tools-item-txt'>等待拼团</Text>
                    </View>
                    <View className='tools-item'>
                      <Image src={require('@assets/shop/banner1.jpg')} className='tools-item-img'></Image>
                      <Text className='tools-item-txt'>等待拼团</Text>
                    </View>
                    <View className='tools-item'>
                      <Image src={require('@assets/shop/banner1.jpg')} className='tools-item-img'></Image>
                      <Text className='tools-item-txt'>等待拼团</Text>
                    </View>
                    <View className='tools-item'>
                      <Image src={require('@assets/shop/banner1.jpg')} className='tools-item-img'></Image>
                      <Text className='tools-item-txt'>等待拼团</Text>
                    </View>
                    <View className='tools-item'>
                      <Image src={require('@assets/shop/banner1.jpg')} className='tools-item-img'></Image>
                      <Text className='tools-item-txt'>等待拼团</Text>
                    </View>
                    <View className='tools-item'>
                      <Image src={require('@assets/shop/banner1.jpg')} className='tools-item-img'></Image>
                      <Text className='tools-item-txt'>等待拼团</Text>
                    </View>
                    <View className='tools-item'>
                      <Image src={require('@assets/shop/banner1.jpg')} className='tools-item-img'></Image>
                      <Text className='tools-item-txt'>等待拼团</Text>
                    </View>
                  </View>
                </View>

                <View className="assistance-text">活动说明</View>
                <Image src={require('@assets/shop/banner1.jpg')}style="width:100%"></Image>
                <Button className={true ? "assistance-btn" : "nostart-btn"}>{true ? "邀请好友拼团" : "未开始"}</Button>
              </ScrollView>


              <View className="chat-entrance" onClick={this.handleMoreClick}>
                <Text className="iconfont icon-chat"></Text>
              </View>
              <BackHomeBtn></BackHomeBtn>
            </View>
          )
        }
      </Block>
    )
  }
}