import Taro, { Component } from '@tarojs/taro';
import './index.scss'
import Notice from '@components/Notice'
import BackHomeBtn from '@components/BackHomeBtn'
import AchievePhone from '@components/achievePhone'
import Poster from './poster'
import dayjs from 'dayjs'
// import WxParse from '@components/wxParse/wxParse'
import getUserPhone from '@utils/getUserPhone'
import ready from '@utils/ready'
import { getDownloadURL } from '@utils/tools'
import {
  getHelpDetail,
  createHelpActivity,
  giveFriendHelp,
} from '@services/activity'
import {
  getMiniQrcode,
  savePoint,
  updatePoint
} from '@services/common'
import { connect } from '@tarojs/redux'

const times = n => n > 0 ? '*'.repeat(n-1).split('*') : []

// 活动状态
const ActBeforeStart = 0;
const ActInProcess = 1;
const ActFinished = 2;

// 发起的助力状态
const HelpInProcess = 1;
const HelpSuccess = 0;
const HelpFailure = 2;

const defaultAvatar = require('@assets/shop/banner1.jpg')

@connect(state => state.user)
export default class Detail extends Component {
  config = {
    navigationBarTitleText: '助力详情'
  }

  state = {
    id: null, // 活动ID
    initiateId: undefined,  // 发起活动
    loaded: false,
    detail: {}, // 活动详情
    initiateDetail: {}, // 发起详情
    helpList: [], // 助力人员列表
    shares: [], // 分享设置
    posters: [],  // 海报设置
    leftTime: 0,  // 剩余时间
    ltTicker: undefined,  // 剩余时间计时器
    actState: ActBeforeStart,  // 活动本身状态
    helpState: HelpInProcess, // 发起助力活动的状态
    isStarter: true, // 是否发起人
    submitting: false, // 是否提交中
    actionSheetVisible: false, // 底部分享好友
    posterVisible: false, // 海报分享
    grantPhoneVisible: false, // 授权电话
    grantAvatarVisible: false, // 授权头像
  }

  componentWillMount() {
    ready.queue(() => {
      // 必须先授权电话
      if (this.toggleGrantPhone()) {
        this.initPageData()
      }
    })
  }

  componentWillUnmount() {
    this.clearTicker()
  }

  initPageData = () => {
    if (!this.state.detail.helpActivityId) {
      const router = Taro.getStorageSync('router')
      const id = this.$router.params.id || router.query.id
      const initiateId = this.$router.params.initiateId || router.query.initiateId
  
      this.setState({ id, initiateId }, () => {
        this.loadDetail()
      })
    }
  }

  clearTicker() {
    if (this.state.ltTicker) {
      clearInterval(this.state.ltTicker)
      this.setState({ ltTicker: undefined })
    }
  }

  // 调起授权电话
  toggleGrantPhone = () => {
    const { userInfo: { person: { phone } } } = this.props
    if (!phone) {
      this.setState({ grantPhoneVisible: true })
      return false
    }

    return true
  }

  // 调起授权头像
  toggleGrantAvatar = () => {
    const { userInfo: { person: { avatarurl } } } = this.props
    if (!avatarurl) {
      this.setState({ grantAvatarVisible: true })
      return false
    }

    return true
  }

  toggleActionVisible = () => {
    this.setState({
      actionSheetVisible: !this.state.actionSheetVisible
    })
  }

  togglePosterVisible = () => {
    this.setState({
      posterVisible: !this.state.posterVisible,
      actionSheetVisible: false
    })
  }

  updateLeftTime(startDate, endDate) {
    const st = dayjs(startDate).valueOf()
    const ed = dayjs(endDate).valueOf()

    this.setState({
      ltTicker: setInterval(() => {
        const nw = dayjs().valueOf()

        if (st > nw) {
          // 活动未开始
          this.setState({
            leftTime: st - nw,
            actState: ActBeforeStart,
          })
        } else if (ed > nw) {
          // 活动未结束
          this.setState({
            leftTime: ed - nw,
            actState: ActInProcess,
          })
        } else {
          // 活动已结束
          this.clearTicker()
          this.setState({
            leftTime: 0,
            actState: ActFinished,
          })
        }
      }, 1000)
    })
  }

  formateLeftTime = () => {
    const nd = 1000 * 24 * 60 * 60;
    const nh = 1000 * 60 * 60;
    const nm = 1000 * 60;
    const ns = 1000;

    const day = Math.floor(this.state.leftTime / nd);
    const hour = Math.floor(this.state.leftTime % nd / nh);
    const min = Math.floor(this.state.leftTime % nd % nh / nm);
    const sec = Math.floor(this.state.leftTime % nd % nh % nm / ns);

    return `${day}天${hour}小时${min}分${sec}秒`
  }

  loadDetail() {
    const { id, initiateId } = this.state
    const { userInfo } = this.props

    Taro.showLoading()
    getHelpDetail(id, initiateId).then(res => {
      this.clearTicker()

      const initiateDetail = res.helpInitiateRecord || {}

      Taro.hideLoading()
      this.setState({
        detail: res.helpActivity,
        initiateDetail,
        helpState: initiateDetail.status === undefined ? HelpInProcess : initiateDetail.status,
        shares: res.shareContentList || [],
        posters: res.postList || [],
        helpList: res.helpRecordList || [],
        loaded: true,
        isStarter: !initiateDetail.personId || userInfo.person.personId === initiateDetail.personId,
      })
      
      this.updateLeftTime(res.helpActivity.startDate, res.helpActivity.endDate)
      // WxParse.wxParse('article', 'html', res.desc, this.$scope, 0)

      // savePoint({
      //   event: 'detail',
      //   eventType: 'help',
      //   propertyName: '助力详情',
      //   buildingId: res.buildingId,
      //   targetId: res.helpActivityId,
      //   data: '{}'
      // }).then(res1 => {
      //   this.setState({
      //     recordId: res1.recordId
      //   })
      // })
    })
  }

  currentPageAndParams() {
    const { detail, initiateDetail } = this.state
    const { userInfo: { person: { personId } } } = this.props

    let queryParams = [
      `id=${detail.helpActivityId}`,
      `from=help_share`,
      `recommender=${personId}`
    ]
    if (initiateDetail.helpRecordInitiateId) {
      queryParams.push(`initiateId=${initiateDetail.helpRecordInitiateId}`)
    }

    const res = [
      'pages/activity/detail/assistance',
      queryParams.join('&'),
    ]

    console.log('--page-params->', res)

    return res
  }

  onShareAppMessage = () => {
    const { shares } = this.state

    if (!shares || !shares.length) {
      Taro.hideShareMenu()
      return {}
    }

    savePoint({
      event: 'share',
      eventType: 'help',
      propertyName: '助力详情分享',
      data: '{}'
    })

    const currentPage = this.currentPageAndParams().join('?')
    return {
      title: shares[0].shareContentTitle,
      path: currentPage,//分享地址
      imageUrl: shares[0].shareContentImg
    }
  }

  // 助力好友
  handleHelp = () => {
    if (this.state.submitting) return

    Taro.showLoading()
    this.setState({ submitting: true }, () => {
      const {detail, initiateDetail} = this.state
      const person = this.props.userInfo.person

      giveFriendHelp({
        buildingId: detail.buildingId,
        helpActivityId: detail.helpActivityId,
        personId: person.personId,
        name: person.name || person.nickname,
        phone: person.tel || person.phone,
        avatarurl: person.avatarurl,
        helpRecordInitiateId: initiateDetail.helpRecordInitiateId,
      }).then((res) => {
        Taro.hideLoading()
        const helpList = this.state.helpList || []
        helpList.push(res.helpRecord)

        this.setState({
          submitting: false,
          helpList,
          initiateDetail: res.helpInitiateRecord,
          helpState: res.helpInitiateRecord.status === undefined ? HelpInProcess : res.helpInitiateRecord.status,
        })

        Taro.showToast({
          title: '助力成功',
          icon: 'success'
        })
      }).catch(err => {
        console.log('err:', err)
        Taro.hideLoading()
        this.setState({ submitting: false })
      })
    })
  }

  // 发起我的助力
  startMine = () => {
    // 必须授权头像
    if (this.toggleGrantAvatar()) {
      const detail = this.state.detail
      const person = this.props.userInfo.person

      return new Promise((resolve, reject) => {
        createHelpActivity({
          buildingId: detail.buildingId,
          helpActivityId: detail.helpActivityId,
          personId: person.personId,
          name: person.name || person.nickname,
          phone: person.tel || person.phone,
          avatarurl: person.avatarurl,
        }).then(res => {
          this.setState({
            initiateDetail: res,
            helpList: [],
            isStarter: true,
            helpState: HelpInProcess,
          }, resolve)
        }).catch(err => {
          console.log('err:', err)
          Taro.showToast({
            title: '发起助力失败',
            icon: 'none'
          })
          reject()
        })
      })
    }

    return Promise.reject()
  }

  // 请求好友助力
  askForHelp = () => {
    // 必须授权头像
    if (this.toggleGrantAvatar()) {
      const initiateDetail = this.state.initiateDetail
      if (!initiateDetail.helpRecordInitiateId) {
        // 如果是未发起的助力, 先以当前人身份发起
        this.startMine().then(() => {
          this.toggleActionVisible()
        })
      } else {
        this.toggleActionVisible()
      }
    }
  }

  handleFavor() {
  }

  getPosterData = () => {
    return new Promise(resolve => {
      const { posterData } = this.state
      if (posterData.qrcode) {
        resolve(posterData)
        return
      }

      const { userInfo: { person } } = this.props
      const { avatarurl, nickname } = person
      const { detail: { posters, createDate } } = this.state
      const [page, scene] = this.currentPageAndParams()
      const payload = { page, scene }

      const _avatarurl = getDownloadURL(avatarurl, 'avatar');
      getMiniQrcode(payload).then(qrcode => {
        let data = {
          qrcode,//小程序二维码
          nickname,//访问者名字
          avatarurl: _avatarurl,//访问者头像
          poster: posters[0].posterImg,
          title: posters[0].posterTitle,//海报标题
          createDate: createDate,//时间
        }
        resolve(data)
      })
    })
  }

  // 开始生成海报
  togglePosterStatus = (flag) => {
    if (flag) {
      this.getPosterData().then(posterData => {
        this.setState({
          posterVisible: !!flag,
          posterData
        })
      })
      // App.zhuge.track('生成助力详情海报')
    } else {
      this.setState({
        posterVisible: !!flag
      })
    }
    savePoint({
      event: 'poster',
      eventType: 'help',
      propertyName: '生成助力详情海报',
      data: '{}'
    }).then(res => {
      console.log('生成助力详情海报')
    })
  }

  hideModal() {
    this.setState({
      canChoose: 'none'
    })
  }

  render() {
    const {
      initiateId,
      initiateDetail,
      detail,
      loaded,
      actState,
      posterVisible,
      posterData,
      isStarter,
      helpList,
      helpState,
      actionSheetVisible,
      grantPhoneVisible,
    } = this.state

    const { userInfo } = this.props
    const avatar = initiateId ? initiateDetail.avatarurl : userInfo.person.avatarurl

    return (
      <Block>
        {/* 生成海报 */}
        {posterVisible && (<Poster data={posterData} toggle={this.togglePosterStatus}></Poster>)}
        
        {
          grantPhoneVisible &&
          <AchievePhone user={this.props.userInfo.person} onSuccess={this.initPageData}></AchievePhone>
        }

        {
          loaded && (
            <View>
              <Notice></Notice>
              <View className="detail-banner">
                <Image mode="widthFix" src={detail.img} className="detail-banner__img"></Image>
                {
                  helpState === HelpInProcess && actState != ActFinished &&
                  (
                    <View className="rest-time">
                      <Text className="row-label">{actState === ActBeforeStart ? '距活动开始:' : (actState === ActInProcess ? '活动剩余时间:' : '活动已结束')} </Text>
                      <Text className="row-content">{actState != ActFinished ? this.formateLeftTime() : ''}</Text>
                    </View>
                  )
                }
              </View>
              <ScrollView
                scrollY
                className="detail-wrap">
                <View className="detail">
                  <View class="detail-title">{detail.title}</View>
                  {
                    helpState === HelpSuccess &&
                    (
                      <Block>
                        <Text>TODO: 助力成功</Text>
                        {
                          isStarter && (<Text>核销码 {initiateDetail.verification}</Text>)
                        }                        
                      </Block>
                    )
                  }
                  {
                    helpState === HelpFailure &&
                    (
                      <Text>TODO: 助力失败</Text>
                    )
                  }
                  <Image src={avatar} className="my__img"></Image>
                  <View className="assistance-text">邀请{detail.personNum}位好友即可赢取大奖</View>
                  <View className="assistance-text">{`${detail.enlistNum || 0}/${detail.personNum}`}</View>
                  <View className='tools-main'>
                    {
                      helpList.map((it, inx) => {
                        return (
                          <View className='tools-item' key={`help-${inx}`}>
                            <Image src={it.avatarurl} className='tools-item-img'></Image>
                            <Text className='tools-item-txt'>{it.name}</Text>
                          </View>
                        )
                      })
                    }
                    {
                      times(detail.personNum - helpList.length).map((_, inx) => {
                        return (
                          <View className='tools-item' key={`un-${inx}`}>
                            <Image src={defaultAvatar} className='tools-item-img'></Image>
                            <Text className='tools-item-txt'>等待助力</Text>
                          </View>
                        )
                      })
                    }
                  </View>
                </View>
                <View>
                  {
                    actState === ActBeforeStart &&
                    (<Button className="nostart-btn">未开始</Button>)
                  }
                  {
                    isStarter && actState === ActInProcess && helpState === HelpInProcess &&
                    (<Button className="assistance-btn" onClick={this.askForHelp}>邀请好友助力</Button>)
                  }
                  {
                    !isStarter && actState === ActInProcess &&
                    (
                      <Block>
                        {
                          helpState === HelpInProcess &&
                          (<Button className="assistance-btn" onClick={this.handleHelp}>立即助力好友</Button>)
                        }
                        <Button className="assistance-btn" onClick={this.startMine}>发起我的助力</Button>
                      </Block>
                    )
                  }
                </View>
                <View className="assistance-text">活动说明</View>
                <Image src={detail.activityInstructions}style="width:100%"></Image>
              </ScrollView>

              <action-sheet hidden={!actionSheetVisible} bindchange={this.toggleActionVisible}>
                <action-sheet-item open-type="share">
                  <Button open-type="share" className='share__friend' onClick={this.toggleActionVisible}>分享给好友</Button>
                </action-sheet-item>
                <action-sheet-item>
                  <Button className='creat__img' onClick={this.togglePosterVisible}>发送朋友圈</Button>
                </action-sheet-item>
                <action-sheet-cancel onClick={this.toggleActionVisible}>取消</action-sheet-cancel>
              </action-sheet>
            
              <BackHomeBtn></BackHomeBtn>              
            </View>
          )
        }
      </Block>
    )
  }
}
